@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import datetime
 
 from django.conf import settings
@@ -7,9 +5,8 @@ from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 from django.http import Http404
 from django.utils import timezone
-from django.utils.encoding import force_str, force_text
 from django.utils.functional import cached_property
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.views.generic.base import View
 from django.views.generic.detail import (
     BaseDetailView, SingleObjectTemplateResponseMixin,
@@ -19,10 +16,8 @@ from django.views.generic.list import (
 )
 
 
-class YearMixin(object):
-    """
-    Mixin for views manipulating year-based data.
-    """
+class YearMixin:
+    """Mixin for views manipulating year-based data."""
     year_format = '%Y'
     year = None
 
@@ -34,9 +29,7 @@ class YearMixin(object):
         return self.year_format
 
     def get_year(self):
-        """
-        Return the year for which this view should display data.
-        """
+        """Return the year for which this view should display data."""
         year = self.year
         if year is None:
             try:
@@ -49,15 +42,11 @@ class YearMixin(object):
         return year
 
     def get_next_year(self, date):
-        """
-        Get the next valid year.
-        """
+        """Get the next valid year."""
         return _get_next_prev(self, date, is_previous=False, period='year')
 
     def get_previous_year(self, date):
-        """
-        Get the previous valid year.
-        """
+        """Get the previous valid year."""
         return _get_next_prev(self, date, is_previous=True, period='year')
 
     def _get_next_year(self, date):
@@ -66,19 +55,18 @@ class YearMixin(object):
 
         The interval is defined by start date <= item date < next start date.
         """
-        return date.replace(year=date.year + 1, month=1, day=1)
+        try:
+            return date.replace(year=date.year + 1, month=1, day=1)
+        except ValueError:
+            raise Http404(_("Date out of range"))
 
     def _get_current_year(self, date):
-        """
-        Return the start date of the current interval.
-        """
+        """Return the start date of the current interval."""
         return date.replace(month=1, day=1)
 
 
-class MonthMixin(object):
-    """
-    Mixin for views manipulating month-based data.
-    """
+class MonthMixin:
+    """Mixin for views manipulating month-based data."""
     month_format = '%b'
     month = None
 
@@ -90,9 +78,7 @@ class MonthMixin(object):
         return self.month_format
 
     def get_month(self):
-        """
-        Return the month for which this view should display data.
-        """
+        """Return the month for which this view should display data."""
         month = self.month
         if month is None:
             try:
@@ -105,15 +91,11 @@ class MonthMixin(object):
         return month
 
     def get_next_month(self, date):
-        """
-        Get the next valid month.
-        """
+        """Get the next valid month."""
         return _get_next_prev(self, date, is_previous=False, period='month')
 
     def get_previous_month(self, date):
-        """
-        Get the previous valid month.
-        """
+        """Get the previous valid month."""
         return _get_next_prev(self, date, is_previous=True, period='month')
 
     def _get_next_month(self, date):
@@ -123,21 +105,20 @@ class MonthMixin(object):
         The interval is defined by start date <= item date < next start date.
         """
         if date.month == 12:
-            return date.replace(year=date.year + 1, month=1, day=1)
+            try:
+                return date.replace(year=date.year + 1, month=1, day=1)
+            except ValueError:
+                raise Http404(_("Date out of range"))
         else:
             return date.replace(month=date.month + 1, day=1)
 
     def _get_current_month(self, date):
-        """
-        Return the start date of the previous interval.
-        """
+        """Return the start date of the previous interval."""
         return date.replace(day=1)
 
 
-class DayMixin(object):
-    """
-    Mixin for views manipulating day-based data.
-    """
+class DayMixin:
+    """Mixin for views manipulating day-based data."""
     day_format = '%d'
     day = None
 
@@ -149,9 +130,7 @@ class DayMixin(object):
         return self.day_format
 
     def get_day(self):
-        """
-        Return the day for which this view should display data.
-        """
+        """Return the day for which this view should display data."""
         day = self.day
         if day is None:
             try:
@@ -164,15 +143,11 @@ class DayMixin(object):
         return day
 
     def get_next_day(self, date):
-        """
-        Get the next valid day.
-        """
+        """Get the next valid day."""
         return _get_next_prev(self, date, is_previous=False, period='day')
 
     def get_previous_day(self, date):
-        """
-        Get the previous valid day.
-        """
+        """Get the previous valid day."""
         return _get_next_prev(self, date, is_previous=True, period='day')
 
     def _get_next_day(self, date):
@@ -184,16 +159,12 @@ class DayMixin(object):
         return date + datetime.timedelta(days=1)
 
     def _get_current_day(self, date):
-        """
-        Return the start date of the current interval.
-        """
+        """Return the start date of the current interval."""
         return date
 
 
-class WeekMixin(object):
-    """
-    Mixin for views manipulating week-based data.
-    """
+class WeekMixin:
+    """Mixin for views manipulating week-based data."""
     week_format = '%U'
     week = None
 
@@ -205,9 +176,7 @@ class WeekMixin(object):
         return self.week_format
 
     def get_week(self):
-        """
-        Return the week for which this view should display data
-        """
+        """Return the week for which this view should display data."""
         week = self.week
         if week is None:
             try:
@@ -220,15 +189,11 @@ class WeekMixin(object):
         return week
 
     def get_next_week(self, date):
-        """
-        Get the next valid week.
-        """
+        """Get the next valid week."""
         return _get_next_prev(self, date, is_previous=False, period='week')
 
     def get_previous_week(self, date):
-        """
-        Get the previous valid week.
-        """
+        """Get the previous valid week."""
         return _get_next_prev(self, date, is_previous=True, period='week')
 
     def _get_next_week(self, date):
@@ -237,12 +202,13 @@ class WeekMixin(object):
 
         The interval is defined by start date <= item date < next start date.
         """
-        return date + datetime.timedelta(days=7 - self._get_weekday(date))
+        try:
+            return date + datetime.timedelta(days=7 - self._get_weekday(date))
+        except OverflowError:
+            raise Http404(_("Date out of range"))
 
     def _get_current_week(self, date):
-        """
-        Return the start date of the current interval.
-        """
+        """Return the start date of the current interval."""
         return date - datetime.timedelta(self._get_weekday(date))
 
     def _get_weekday(self, date):
@@ -260,24 +226,20 @@ class WeekMixin(object):
             raise ValueError("unknown week format: %s" % week_format)
 
 
-class DateMixin(object):
-    """
-    Mixin class for views manipulating date-based data.
-    """
+class DateMixin:
+    """Mixin class for views manipulating date-based data."""
     date_field = None
     allow_future = False
 
     def get_date_field(self):
-        """
-        Get the name of the date field to be used to filter by.
-        """
+        """Get the name of the date field to be used to filter by."""
         if self.date_field is None:
             raise ImproperlyConfigured("%s.date_field is required." % self.__class__.__name__)
         return self.date_field
 
     def get_allow_future(self):
         """
-        Returns `True` if the view should be allowed to display objects from
+        Return `True` if the view should be allowed to display objects from
         the future.
         """
         return self.allow_future
@@ -305,7 +267,7 @@ class DateMixin(object):
         if self.uses_datetime_field:
             value = datetime.datetime.combine(value, datetime.time.min)
             if settings.USE_TZ:
-                value = timezone.make_aware(value, timezone.get_current_timezone())
+                value = timezone.make_aware(value)
         return value
 
     def _make_single_date_lookup(self, date):
@@ -329,28 +291,26 @@ class DateMixin(object):
 
 
 class BaseDateListView(MultipleObjectMixin, DateMixin, View):
-    """
-    Abstract base class for date-based views displaying a list of objects.
-    """
+    """Abstract base class for date-based views displaying a list of objects."""
     allow_empty = False
     date_list_period = 'year'
 
     def get(self, request, *args, **kwargs):
         self.date_list, self.object_list, extra_context = self.get_dated_items()
-        context = self.get_context_data(object_list=self.object_list,
-                                        date_list=self.date_list)
-        context.update(extra_context)
+        context = self.get_context_data(
+            object_list=self.object_list,
+            date_list=self.date_list,
+            **extra_context
+        )
         return self.render_to_response(context)
 
     def get_dated_items(self):
-        """
-        Obtain the list of dates and items.
-        """
+        """Obtain the list of dates and items."""
         raise NotImplementedError('A DateView must provide an implementation of get_dated_items()')
 
     def get_ordering(self):
         """
-        Returns the field or fields to use for ordering the queryset; uses the
+        Return the field or fields to use for ordering the queryset; use the
         date field by default.
         """
         return '-%s' % self.get_date_field() if self.ordering is None else self.ordering
@@ -373,17 +333,18 @@ class BaseDateListView(MultipleObjectMixin, DateMixin, View):
         if not allow_empty:
             # When pagination is enabled, it's better to do a cheap query
             # than to load the unpaginated queryset in memory.
-            is_empty = len(qs) == 0 if paginate_by is None else not qs.exists()
+            is_empty = not qs if paginate_by is None else not qs.exists()
             if is_empty:
                 raise Http404(_("No %(verbose_name_plural)s available") % {
-                    'verbose_name_plural': force_text(qs.model._meta.verbose_name_plural)
+                    'verbose_name_plural': qs.model._meta.verbose_name_plural,
                 })
 
         return qs
 
     def get_date_list_period(self):
         """
-        Get the aggregation period for the list of dates: 'year', 'month', or 'day'.
+        Get the aggregation period for the list of dates: 'year', 'month', or
+        'day'.
         """
         return self.date_list_period
 
@@ -402,25 +363,23 @@ class BaseDateListView(MultipleObjectMixin, DateMixin, View):
         else:
             date_list = queryset.dates(date_field, date_type, ordering)
         if date_list is not None and not date_list and not allow_empty:
-            name = force_text(queryset.model._meta.verbose_name_plural)
-            raise Http404(_("No %(verbose_name_plural)s available") %
-                          {'verbose_name_plural': name})
+            raise Http404(
+                _("No %(verbose_name_plural)s available") % {
+                    'verbose_name_plural': queryset.model._meta.verbose_name_plural,
+                }
+            )
 
         return date_list
 
 
 class BaseArchiveIndexView(BaseDateListView):
     """
-    Base class for archives of date-based items.
-
-    Requires a response mixin.
+    Base class for archives of date-based items. Requires a response mixin.
     """
     context_object_name = 'latest'
 
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         qs = self.get_dated_queryset()
         date_list = self.get_date_list(qs, ordering='DESC')
 
@@ -431,23 +390,17 @@ class BaseArchiveIndexView(BaseDateListView):
 
 
 class ArchiveIndexView(MultipleObjectTemplateResponseMixin, BaseArchiveIndexView):
-    """
-    Top-level archive of date-based items.
-    """
+    """Top-level archive of date-based items."""
     template_name_suffix = '_archive'
 
 
 class BaseYearArchiveView(YearMixin, BaseDateListView):
-    """
-    List of objects published in a given year.
-    """
+    """List of objects published in a given year."""
     date_list_period = 'month'
     make_object_list = False
 
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         year = self.get_year()
 
         date_field = self.get_date_field()
@@ -483,22 +436,16 @@ class BaseYearArchiveView(YearMixin, BaseDateListView):
 
 
 class YearArchiveView(MultipleObjectTemplateResponseMixin, BaseYearArchiveView):
-    """
-    List of objects published in a given year.
-    """
+    """List of objects published in a given year."""
     template_name_suffix = '_archive_year'
 
 
 class BaseMonthArchiveView(YearMixin, MonthMixin, BaseDateListView):
-    """
-    List of objects published in a given month.
-    """
+    """List of objects published in a given month."""
     date_list_period = 'day'
 
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         year = self.get_year()
         month = self.get_month()
 
@@ -524,21 +471,15 @@ class BaseMonthArchiveView(YearMixin, MonthMixin, BaseDateListView):
 
 
 class MonthArchiveView(MultipleObjectTemplateResponseMixin, BaseMonthArchiveView):
-    """
-    List of objects published in a given month.
-    """
+    """List of objects published in a given month."""
     template_name_suffix = '_archive_month'
 
 
 class BaseWeekArchiveView(YearMixin, WeekMixin, BaseDateListView):
-    """
-    List of objects published in a given week.
-    """
+    """List of objects published in a given week."""
 
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         year = self.get_year()
         week = self.get_week()
 
@@ -569,20 +510,14 @@ class BaseWeekArchiveView(YearMixin, WeekMixin, BaseDateListView):
 
 
 class WeekArchiveView(MultipleObjectTemplateResponseMixin, BaseWeekArchiveView):
-    """
-    List of objects published in a given week.
-    """
+    """List of objects published in a given week."""
     template_name_suffix = '_archive_week'
 
 
 class BaseDayArchiveView(YearMixin, MonthMixin, DayMixin, BaseDateListView):
-    """
-    List of objects published on a given day.
-    """
+    """List of objects published on a given day."""
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         year = self.get_year()
         month = self.get_month()
         day = self.get_day()
@@ -611,28 +546,20 @@ class BaseDayArchiveView(YearMixin, MonthMixin, DayMixin, BaseDateListView):
 
 
 class DayArchiveView(MultipleObjectTemplateResponseMixin, BaseDayArchiveView):
-    """
-    List of objects published on a given day.
-    """
+    """List of objects published on a given day."""
     template_name_suffix = "_archive_day"
 
 
 class BaseTodayArchiveView(BaseDayArchiveView):
-    """
-    List of objects published today.
-    """
+    """List of objects published today."""
 
     def get_dated_items(self):
-        """
-        Return (date_list, items, extra_context) for this request.
-        """
+        """Return (date_list, items, extra_context) for this request."""
         return self._get_dated_items(datetime.date.today())
 
 
 class TodayArchiveView(MultipleObjectTemplateResponseMixin, BaseTodayArchiveView):
-    """
-    List of objects published today.
-    """
+    """List of objects published today."""
     template_name_suffix = "_archive_day"
 
 
@@ -642,9 +569,7 @@ class BaseDateDetailView(YearMixin, MonthMixin, DayMixin, DateMixin, BaseDetailV
     standard DetailView by accepting a year/month/day in the URL.
     """
     def get_object(self, queryset=None):
-        """
-        Get the object this request displays.
-        """
+        """Get the object this request displays."""
         year = self.get_year()
         month = self.get_month()
         day = self.get_day()
@@ -670,7 +595,7 @@ class BaseDateDetailView(YearMixin, MonthMixin, DayMixin, DateMixin, BaseDetailV
         lookup_kwargs = self._make_single_date_lookup(date)
         qs = qs.filter(**lookup_kwargs)
 
-        return super(BaseDetailView, self).get_object(queryset=qs)
+        return super().get_object(queryset=qs)
 
 
 class DateDetailView(SingleObjectTemplateResponseMixin, BaseDateDetailView):
@@ -683,13 +608,13 @@ class DateDetailView(SingleObjectTemplateResponseMixin, BaseDateDetailView):
 
 def _date_from_string(year, year_format, month='', month_format='', day='', day_format='', delim='__'):
     """
-    Helper: get a datetime.date object given a format string and a year,
-    month, and day (only year is mandatory). Raise a 404 for an invalid date.
+    Get a datetime.date object given a format string and a year, month, and day
+    (only year is mandatory). Raise a 404 for an invalid date.
     """
-    format = delim.join((year_format, month_format, day_format))
-    datestr = delim.join((year, month, day))
+    format = year_format + delim + month_format + delim + day_format
+    datestr = str(year) + delim + str(month) + delim + str(day)
     try:
-        return datetime.datetime.strptime(force_str(datestr), format).date()
+        return datetime.datetime.strptime(datestr, format).date()
     except ValueError:
         raise Http404(_("Invalid date string '%(datestr)s' given format '%(format)s'") % {
             'datestr': datestr,
@@ -699,9 +624,9 @@ def _date_from_string(year, year_format, month='', month_format='', day='', day_
 
 def _get_next_prev(generic_view, date, is_previous, period):
     """
-    Helper: Get the next or the previous valid date. The idea is to allow
-    links on month/day views to never be 404s by never providing a date
-    that'll be invalid for the given view.
+    Get the next or the previous valid date. The idea is to allow links on
+    month/day views to never be 404s by never providing a date that'll be
+    invalid for the given view.
 
     This is a bit complicated since it handles different intervals of time,
     hence the coupling to generic_view.
@@ -788,9 +713,7 @@ def _get_next_prev(generic_view, date, is_previous, period):
 
 
 def timezone_today():
-    """
-    Return the current date in the current time zone.
-    """
+    """Return the current date in the current time zone."""
     if settings.USE_TZ:
         return timezone.localdate()
     else:
